/*  brlkey
    Copyright(C) 2008 PaulLiu(Ying-Chun Liu). All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <iostream>
#include <vector>
#include <stack>
#include <map>
#include <sstream>
#include <fstream>
#include <algorithm>

#include <config.h>
#include <cstdio>
#include <cstdlib>
#include <cstring>

#include <unistd.h>
#include <pty.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>

#include <linux/input.h>

#include "common.h"

using namespace std;


vector < vector < string > >liu_key;
vector < string > liu_output;

int
loadliutable (void)
/* load liu translation table into memory */
{
  char com1[1000], com2[1000];
  ifstream fin;
  string filename, s, s1, s2, s3, s21;
  istringstream iss;
  vector < string > tt1;

  /* open data file */
  filename = string (PKGDATADIR);
  filename += "/liutable.txt";

  fin.open (filename.c_str (), ios::in);
  if (fin.fail ())
    {
      cerr << "Error: cannot open " << filename << endl;
      return -1;
    }

  while (!fin.eof ())
    {
      /* read a line */
      fin.getline (com1, sizeof (com1) - 1);
      if (fin.fail ())
	break;
      s = string (com1);
      if (s[0] == '#')
	continue;		/* comment */
      iss.clear ();
      iss.str (s);
      iss >> s1 >> s2 >> s3;	/* s1: no use, s2: key, s3: liu */
      if (iss.fail ())
	continue;
      strcpy (com2, s3.c_str ());
      if(s3=="\\040") toprintfstring (com2);
      liu_output.push_back (string (com2));
      replace (s2.begin (), s2.end (), '-', ' ');
      iss.clear ();
      iss.str (s2.c_str ());
      for (tt1.clear (); !iss.eof ();)
	{
	  iss >> s21;
	  if (iss.fail ())
	    break;
	  tt1.push_back (s21);
	}
      liu_key.push_back (tt1);
    }
  return 0;
}

int
liu_trans (const int *keyboard_array, int keyboard_count)
{
  int mykbar[128], mykbar_count = 0;
  int i, j, flag, soundflag;
  static int liu_now = 0;
  static vector < int >v1 (liu_key.size (), 0);

  soundflag = 0;
  for (; liu_now >= 0; liu_now--)
    {
      flag = 0;
      for (i = 0; i < (int) liu_key.size (); i++)
	{
	  if (liu_now >= (int) liu_key[i].size ())
	    continue;
	  memset (mykbar, 0, sizeof (mykbar));
	  mykbar_count = genarray (liu_key[i][liu_now].c_str (), mykbar);
	  if (keyboard_count == mykbar_count
	      && memcmp (mykbar, keyboard_array, sizeof (mykbar)) == 0)
	    {
	      if (liu_now == v1[i])
		{
		  v1[i]++;
		  flag = 1;
		}
	    }
	}
      if (flag == 1)
	{
	  liu_now++;
	  break;
	}
      else
	{
	  soundflag = 1;
	  for (j = 0; j < (int) v1.size (); j++)
	    {
	      if (v1[j] == liu_now)
		v1[j]--;
	    }
	}
    }
  if (soundflag == 1)
    {
      playsound ("liuerror");
    }
  if (liu_now < 0)
    {
      liu_now = 0;
      v1 = vector < int >(liu_key.size (), 0);
    }
  for (i = 0; i < (int) liu_key.size (); i++)
    {
      if (v1[i] == (int) liu_key[i].size ())
	{
	  output_string (liu_output[i].c_str ());
	  v1 = vector < int >(liu_key.size (), 0);
	  liu_now = 0;
	  break;
	}
    }
  return liu_now;
}

int
liu_test1 ()
{
  for (int i = 0; i < (int) liu_key.size (); i++)
    {
      for (int j = 0; j < (int) liu_key[i].size (); j++)
	{
	  if (j > 0)
	    cout << "-";
	  cout << liu_key[i][j];
	}
      cout << " " << liu_output[i] << endl;
    }
  return 0;
}

int
check_liukey (const int *keys)
/* return value: true - pass, false - fail */
{
  int i;
  for (i = 0; i < KEYARRAYSIZE; i++) {
    if (keys[i])
      if (!((i >= KEY_A && i <= KEY_F) ||
	    (i >= KEY_J && i <= KEY_SEMICOLON) ||
	     i == KEY_SPACE))
	return 0;
  }
  return 1;
}

int
liu_event_handler (const struct keyevent *ev)
{
  int now;

  now = (*ev).key;
  switch ((*ev).type)
    {
    case KEYEVENT_RELEASE:
    case KEYEVENT_PRESS:
      if (isnormalkey (now) || (*ev).keyarray[KEY_LEFTALT]
	  || (*ev).keyarray[KEY_RIGHTALT] || (*ev).keyarray[KEY_LEFTCTRL]
	  || (*ev).keyarray[KEY_RIGHTCTRL] || (*ev).keyarray[KEY_LEFTSHIFT] || (*ev).keyarray[KEY_RIGHTSHIFT])
	{
	  struct input_event event = {};
	  event.type = EV_KEY;
	  event.code = now;
	  event.value = (*ev).type == KEYEVENT_PRESS;
	  if (normalhandler(ev) == 0)
	    output_event (&event);
	}
      break;
    case KEYEVENT_ACCUMULATION:
      if (check_liukey ((*ev).keyarray))
	{
	  liu_trans ((*ev).keyarray, (*ev).keyarray_count);
	}
      else
	{
	  /* include libchewing ?? */
	}
      break;
    default:
      break;
    }
  return 0;
}

int
liu_main ()
{
  loadliutable ();
  liu_test1 ();
  return 0;
}
