/*  brlkey
    Copyright(C) 2008 PaulLiu(Ying-Chun Liu). All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <config.h>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <ctime>

#include <termios.h>
#include <unistd.h>
#include <pty.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>

#include "common.h"

using namespace std;

char shellname[] = { "/bin/bash" };
int shellfd = -1;
int start=0;

int
reverse_hex (char a)
/* char hex -> int decimal */
{
  static char tb1[] = { "0123456789abcdef" };
  int i;

  for (i = 0; i < (int) sizeof (tb1); i++)
    {
      if (tb1[i] == a)
	{
	  return i;
	}
    }
  return -1;
}

int
toprintfstring (char *com1)
/* \\xXX \\0XX -> char */
{
  char *s;
  char com2[4];
  int i;
  for (s = com1; *s != '\0'; s++, com1++)
    {
      if (*s == '\\')
	{
	  for (i = 0; i < 3; i++)
	    {
	      s++;
	      com2[i] = *s;
	    }
	  com2[i] = '\0';
	  if (com2[0] != 'x')	/* oct */
	    {
	      *com1 = (com2[0] - '0') * '\100'
		+ (com2[1] - '0') * '\010' + (com2[2] - '0') * '\001';
	    }
	  else			/* hex */
	    {
	      *com1 =
		(char) (reverse_hex (com2[1]) * 16 + reverse_hex (com2[2]));
	    }
	}
      else
	{
	  *com1 = *s;
	}
    }
  *com1 = '\0';
  return 0;
}


int
handleopt (int argc, char *argv[])
{
  int c,ret;
  char progname[1000];
  strncpy(progname,argv[0],sizeof(progname)-1);
  ret=0;
  c=getopt(argc,argv,"vhq");
  switch(c)
  {
    case 'v':
      printf("%s\n", VERSION);
      ret=1;
      break;
    case 'h':
      printf("Usage: %s [-h] [-v] [-q]\n",progname);
      printf("-h: Show this page\n");
      printf("-v Show version only\n");
      printf("-q: Start at QWERTY mode\n");
      ret=1;
      break;
    case 'q':
      start=1;
      break;
  }
  return ret;
}

int
main (int argc, char *argv[])
/* main function */
{
  if(handleopt (argc, argv) != 0)
  {
    exit (0);
  }

  pid_t pid=fork();
  if(pid) exit(0);

  printf("8-dots Braille Keyboard emulator %s by PaulLiu, Coscell & Samuel\n", VERSION);
  printf("left win key = braille input switch.\n");
  printf("space+jkl = Phonetic Chinese input switch.\n");
  printf("space+fds = Boshamy Chinese input switch.\n");
  printf("Excute \"killall brlkey\" to release program.\n");

  /* load tables */
  readscancode ();
  if (linux_init ())
    return 1;
  readiso8859table1 ();
  loadbig5table ();
  loadliutable ();
  loadsoundfile ();

  iohandler ();		/* io handling */
  wait (NULL);
  return 0;
}
